# this is not really a roster, it's a local user database.

package Quak::Roster;

use strict;
use warnings;

use Quak::Utilities;

our @ISA = qw/Exporter/;
our @EXPORT = qw/ROSTER_OFFLINE ROSTER_ONLINE ROSTER_SUBSCRIBED_OFFLINE ROSTER_SUBSCRIBED_ONLINE/;

use constant {
	ROSTER_OFFLINE => 0,
	ROSTER_ONLINE => 1,
	ROSTER_SUBSCRIBED_OFFLINE => 10,
	ROSTER_SUBSCRIBED_ONLINE => 11
};

sub new {
	$::gdbh->do(
		"create table if not exists roster (jid varchar(255) primary key, online int(1), subscribed int(1))"
	);
	return db_err if $::gdbh->err;
	return bless {}, $_[0]
}

sub refresh_online {
	$::gdbh->{'AutoCommit'} = 0;
	$::gdbh->{'RaiseError'} = 1;
	eval {
		$::gdbh->do("update roster set online = 0");
		my $sth = $::gdbh->prepare(
			"insert or replace into roster (jid, online, subscribed) values (?, 1, coalesce((select subscribed from roster where jid = ?),0))"
		);
		$sth->execute(($_ =~ m!^([^/]+)!)[0], ($_ =~ m!^([^/]+)!)[0]) for @{$_[1]};
		$::gdbh->commit
	};
	if ($@) {
		printf "Roster error: %s\n", $@;
		eval {$::gdbh->rollback};
	}
	$::gdbh->{'AutoCommit'} = 1
}

sub subscribe {
	my ($jid) = $_[1] =~ m!^([^/]+)!;
	my $sth = $::gdbh->prepare(
		"insert or replace into roster (jid, online, subscribed) values (?, 1, 1)"
	);
	$sth->execute($jid);
	return ($::gdbh->err)?db_err:1
}

sub unsubscribe {
	my ($jid) = $_[1] =~ m!^([^/]+)!;
	my $sth = $::gdbh->prepare("update roster set subscribed = 0 where jid = ?");
	$sth->execute($jid);
	return ($::gdbh->err)?db_err:1
}

sub online {
	my ($jid) = $_[1] =~ m!^([^/]+)!;
	my $sth;
	if (defined $_[2] && $_[2] =~ /^([01])$/) {
		if ($1 == 0) {
			$sth=$::gdbh->prepare("update roster set online = 0 where jid = ?");
			$sth->execute($jid);
			return ($::gdbh->err)?db_err:1
		}
		$sth = $::gdbh->prepare(
			"insert or replace into roster (jid, online, subscribed) values (?, 1, coalesce((select subscribed from roster where jid = ?),0))"
		);
		$sth->execute($jid, $jid);
		return ($::gdbh->err)?db_err:1
	}
	$sth = $::gdbh->prepare("select jid, online, subscribed from roster where jid = ?");
	$sth->execute($jid);
	my $res = $sth->fetchrow_hashref;
	if (exists $res->{'jid'}) {
		return ROSTER_SUBSCRIBED_ONLINE if $res->{'online'} && $res->{'subscribed'};
		return ROSTER_ONLINE if $res->{'online'};
		return ROSTER_SUBSCRIBED_OFFLINE if $res->{'subscribed'}
	}
	return ROSTER_OFFLINE
}

sub subscribed_jids {
	my $res = $::gdbh->selectall_arrayref(
		"select jid from roster where subscribed = 1",
		{Slice => {}}
	);
	my @ret; foreach my $r (@{$res}) {push @ret, $r->{'jid'}}
	return @ret
}

sub online_subscribed_jids {
	my $res = $::gdbh->selectall_arrayref(
		"select jid from roster where online = 1 and subscribed = 1",
		{Slice => {}}
	);
	my @ret; foreach my $r (@{$res}) {push @ret, $r->{'jid'}}
	return @ret
}

sub online_jids {
	my $res = $::gdbh->selectall_arrayref(
		"select jid from roster where online = 1",
		{Slice => {}}
	);
	my @ret; foreach my $r (@{$res}) {push @ret, $r->{'jid'}}
	return @ret
}

sub escape {
	my $sth=$::gdbh->do("update roster set online = 0");
	return ($::gdbh->err)?db_err:1
}

1;

__END__

=head1 NAME

Quak::Roster - a fake roster for the jabber component

=head1 SYNOPSIS

 use Quak::Roster

 my $roster = new Quak::Roster;

=head1 DESCRIPTION

 This is not really a roster, just a local user database.

 Documentation yet incomplete.

=head1 METHODS

=over 8

=item B<new>()

bless a new object, open database and create table if needed.

=item B<refresh_online>()

refresh database.

=item B<subscribe>

subscribe I<jid>.

=item B<unsubscribe>

unsubscribe I<jid>.

=item B<online>(I<jid>)

returns online status of given I<jid>.

=item B<subscribed_jids>()

returns a list of subscribed jids.

=item B<online_subscribed_jids>()

returns a list of online and subscribed jids.

=item B<online_jids>()

returns a list of online jids.

=item B<escape>()

set online status of all saved jids to '0'.

=back

=head1 CONSTANTS

=over 8

=item B<ROSTER_OFFLINE>

user is offline.

=item B<ROSTER_ONLINE>

user is online.

=item B<ROSTER_SUBSCRIBED_OFFLINE>

user is subscribed but offline.

=item B<ROSTER_SUBSCRIBED_ONLINE>

user is subscribed and online.

=back

=head1 EXAMPLES

...

=head1 AUTHOR

Philippe Kueck <projects at unixadm dot org>

=cut


